package runListener;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.junit.runner.Description;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;
import org.junit.runner.notification.RunListener;

/**
 * {@link org.junit.runner.notification.RunListener} to generate an output file whose existence can be checked by surefire-integration.
 *
 * @author <a href="mailto:matthew.gilliard@gmail.com">Matthew Gilliard</a>
 */
public class EchoingRunListener
    extends RunListener
{

    @Override
    public void testRunStarted( Description description )
        throws Exception
    {
        System.out.println("testRunStarted " + description);
    }

    @Override
    public void testRunFinished( Result result )
        throws Exception
    {
        System.out.println("testRunFinished " + result);
    }

    @Override
    public void testStarted( Description description )
        throws Exception
    {
        System.out.println("testStarted " + description);
    }

    @Override
    public void testFinished( Description description )
        throws Exception
    {
        System.out.println("testFinished " + description);
    }

    @Override
    public void testFailure( Failure failure )
        throws Exception
    {
        System.out.println("testFailure " + failure);
    }

    @Override
    public void testIgnored( Description description )
        throws Exception
    {
        System.out.println("testIgnored " + description);
    }

    public void testAssumptionFailure( Failure failure )
    {
        System.out.println("testAssumptionFailure " + failure);
     }
}
