package argLine;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.junit.Assert;
import org.junit.Test;

public class TestSurefireArgLineProperties
{
    @Test
    public void testFromProp()
    {
        String fromProp = System.getProperty("p1");
        Assert.assertNotNull(fromProp, "incorrect arg line, no p1 present?");
        Assert.assertEquals("from-prop-value", fromProp);
    }

    @Test
    public void testOverrideProp()
    {
        String overrideProp = System.getProperty("p2");
        Assert.assertNotNull(overrideProp, "incorrect arg line, no p2 present?");
        Assert.assertEquals("override-prop-value", overrideProp);
    }

    @Test
    public void testUndefinedProp()
    {
        String undefinedProp = System.getProperty("p3");
        Assert.assertNotNull(undefinedProp, "incorrect arg line, no p3 present?");
        Assert.assertEquals("@{undefined.prop}", undefinedProp);
    }

    @Test
    public void testGeneratedProp()
    {
        String generatedProp = System.getProperty("p4");
        Assert.assertNotNull(generatedProp, "incorrect arg line, no p4 present?");
        Assert.assertEquals("generated-prop-value", generatedProp);
    }
}
