package largeTestResults;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

public class BasicTest
    extends TestCase
{

    private final int number;

    public BasicTest( String name, int number )
    {
        super( name );
        this.number = number;
    }

    public static Test suite()
    {
        int tests = Integer.parseInt( System.getProperty( "numTests", "20" ) );
        TestSuite suite = new TestSuite();
        for ( int i = 0; i < tests; i++ )
        {
            if ( i % 4 == 0 )
            {
                suite.addTest( new BasicTest( "testPass", i ) );
            }
            else
            {
                suite.addTest( new BasicTest( "testFail", i ) );
            }
        }
        return suite;
    }

    public void testFail()
    {
        fail( "failure " + number );
    }

    public void testPass()
    {
    }

}
