import org.junit.Test;

public class J9Test
{
    @Test
    public void test_sql_mod() throws java.sql.SQLException
    {
        System.out.println( System.getProperty( "java.specification.version" ) );
    }

    @Test
    public void test_corba_mod() /*throws org.omg.CORBA.BAD_INV_ORDER*/
    {
    }
}
