package junit.notExtendingTestCase;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import junit.framework.Test;
import junit.framework.TestResult;
import junit.framework.TestSuite;


public class SuiteTest
    extends TestSuite
{
    public static Test suite() {
        SuiteTest suite = new SuiteTest();
        suite.addTest( new Test() {

            public int countTestCases()
            {
                return 1;
            }

            public void run( TestResult result )
            {
                result.startTest( this );
                result.endTest( this );
                
            }
            
        } );
        return suite;
    }
}
